using Ato.CD.Inbound.Shared;
using Ato.EN.IntegrationServices.CodeGenerationCGTS;
using Ato.EN.IntegrationServices.CodeGenerationDISTBENTRT;
using Ato.EN.IntegrationServices.CodeGenerationFTER;
using Ato.EN.IntegrationServices.CodeGenerationIDS;
using Ato.EN.IntegrationServices.CodeGenerationIEE;
using Ato.EN.IntegrationServices.CodeGenerationLS;
using Ato.EN.IntegrationServices.CodeGenerationPSS;
using Ato.EN.IntegrationServices.CodeGenerationRS;
using Ato.EN.IntegrationServices.CodeGenerationTRT;
using DataContracts;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using VaTS;


namespace Ato.CD.Inbound.TRT202502
{
    public class CrossFormValidator
    {
        private IEnumerable<CGTS2018> cgtsChildDocuments { get; set; }
        private IEnumerable<DISTBENTRT2024> distbentrtChildDocuments { get; set; }
        private IEnumerable<FTER2021> fterChildDocuments { get; set; }
        private IEnumerable<IDS2025> idsChildDocuments { get; set; }
        private IEnumerable<IEE2021> ieeChildDocuments { get; set; }
        private IEnumerable<LS2017> lsChildDocuments { get; set; }
        private IEnumerable<PSS2018> pssChildDocuments { get; set; }
        private IEnumerable<RS2018> rsChildDocuments { get; set; }

        public List<ProcessMessageDocument> response { get; private set; }
        private TRT2025 parentDocument { get; set; }

        public CrossFormValidator(TRT2025 parent, BusinessDocumentCollection childDocuments)
        {
            response = new List<ProcessMessageDocument>();
            parentDocument = parent;
            BusinessDocumentCollection ChildDocuments = childDocuments ?? new BusinessDocumentCollection();

            cgtsChildDocuments = ChildDocuments
                .Where(s => s.DocumentName.Equals(Schedule.CGTS.ToString()))
                .Select(p => (CGTS2018)p.ConsumedReport);

            distbentrtChildDocuments = ChildDocuments
                .Where(s => s.DocumentName.Equals(Schedule.DISTBENTRT.ToString()))
                .Select(p => (DISTBENTRT2024)p.ConsumedReport);

            fterChildDocuments = ChildDocuments
                .Where(s => s.DocumentName.Equals(Schedule.FTER.ToString()))
                .Select(p => (FTER2021)p.ConsumedReport);

            idsChildDocuments = ChildDocuments
                .Where(s => s.DocumentName.Equals(Schedule.IDS.ToString()))
                .Select(p => (IDS2025)p.ConsumedReport);

            ieeChildDocuments = ChildDocuments
                .Where(s => s.DocumentName.Equals(Schedule.IEE.ToString()))
                .Select(p => (IEE2021)p.ConsumedReport);

            lsChildDocuments = ChildDocuments
                .Where(s => s.DocumentName.Equals(Schedule.LS.ToString()))
                .Select(p => (LS2017)p.ConsumedReport);

            pssChildDocuments = ChildDocuments
                .Where(s => s.DocumentName.Equals(Schedule.PSS.ToString()))
                .Select(p => (PSS2018)p.ConsumedReport);

            rsChildDocuments = ChildDocuments
                .Where(s => s.DocumentName.Equals(Schedule.RS.ToString()))
                .Select(p => (RS2018)p.ConsumedReport);
        }

        public List<ProcessMessageDocument> ValidateCrossFormRules()
        {
            //CGTS child document cross form validation
            VRATOTRT432042();

            foreach (var cgtsChildDocument in cgtsChildDocuments)
            {
                VRATOTRT432189(cgtsChildDocument);
                VRATOTRT432213(cgtsChildDocument);
                VRATOTRT433043(cgtsChildDocument);
                VRATOTRT433044(cgtsChildDocument);
                VRATOTRT433045(cgtsChildDocument);
                VRATOTRT433046(cgtsChildDocument);
                VRATOTRT433047(cgtsChildDocument);
                VRATOTRT433048(cgtsChildDocument);
                VRATOTRT433049(cgtsChildDocument);
            }

            //DISTBENTRT child document cross form validation
            VRATOTRT442897();

            foreach (var distbentrtChildDocument in distbentrtChildDocuments)
            {
                VRATOTRT442894(distbentrtChildDocument);
                VRATOTRT442895(distbentrtChildDocument);
                VRATOTRT442896(distbentrtChildDocument);
                VRATOTRTW00021(distbentrtChildDocument);
            }

            //FTER child document cross form validation
            VRATOTRT432017();

            //IDS child document cross form validation
            VRATOTRT432648();
            VRATOTRT432675();
            VRATOTRT432676();
            VRATOTRT433001();
            VRATOTRT433050();

            foreach (var idsChildDocument in idsChildDocuments)
            {
                VRATOTRT432646(idsChildDocument);
                VRATOTRT432647(idsChildDocument);
                VRATOTRT432649(idsChildDocument);
                VRATOTRT432650(idsChildDocument);
                VRATOTRT432651(idsChildDocument);
                VRATOTRT432652(idsChildDocument);
                VRATOTRT432653(idsChildDocument);
                VRATOTRT432655(idsChildDocument);
                VRATOTRT432656(idsChildDocument);
                VRATOTRT432707(idsChildDocument);
                VRATOTRT432720(idsChildDocument);
                VRATOTRT432797(idsChildDocument);
            }

            //IEE child document cross form validation
            VRATOTRT432011();
            VRATOTRT432019();
            VRATOTRT432021();
            VRATOTRT432927();

            //LS child document cross form validation
            VRATOTRT432210();
            VRATOTRT432429();

            foreach (var lsChildDocument in lsChildDocuments)
            {
                VRATOTRT432208(lsChildDocument);
                VRATOTRT432212(lsChildDocument);
                VRATOTRT432215(lsChildDocument);
            }

            //PSS child document cross form validation
            VRATOTRT432043();
            VRATOTRT432044();
            VRATOTRT432051();
            VRATOTRT432088();

            //RS child document cross form validation
            VRATOTRT432115();
            VRATOTRT432116();
            VRATOTRT432117();
            VRATOTRT432120();
            VRATOTRT432122();
            VRATOTRT432124();
            VRATOTRT432126();

            return response;
        }

        #region CGTS CrossForm Validation Rules

        #region VR.ATO.TRT.432189

        /*  VR.ATO.TRT.432189
        The Net Capital gain amount on the main return form must equal the Total net capital gain amount on the Capital Gains Tax schedule.

        Legacy Rule Format:
        CountDocument('CGTS') = 1 AND ^TRT133 <> ^CGTS131

        Technical Business Rule Format:
        CountDocument('CGTS') = 1 AND ^TRT133 <> ^CGTS131

        Data Elements:
    
        ^TRT133 = TRT:RP:BusinessDetails:Income:CapitalGains:Income.CapitalGainsNet.Amount
    
        ^CGTS131 = CGTS:RP:Income.CapitalGainsNet.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432189(CGTS2018 childDocument)
        {
            bool assertion = parentDocument.TRT133.GetValueOrDefault() != childDocument.CGTS131.GetValueOrDefault();

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.434036",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital gains in return must equal total net capital gain in CGT schedule.",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:CapitalGains/tns:NetA",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432189" },
                        new ProcessMessageParameter() { Name = "TRT133", Value = parentDocument.TRT133.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "CGTS131", Value = childDocument.CGTS131.GetValueOrDefault().ToString() },
                    },
                });
            }
        }
        #endregion // VR.ATO.TRT.432189

        #region VR.ATO.TRT.432213

        /*  VR.ATO.TRT.432213
        Unapplied net capital losses carried forward on the Capital Gains Tax schedule (if lodged) must equal Net capital losses carried forward on the main income tax return.

        Legacy Rule Format:
        CountDocument('CGTS') = 1 AND (^CGTS124 + ^CGTS125) <> ^TRT475

        Technical Business Rule Format:
        CountDocument('CGTS') = 1 AND (^CGTS124 + ^CGTS125) <> ^TRT475

        Data Elements:

        ^CGTS124 = CGTS:RP.Collectables.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount

        ^CGTS125 = CGTS:RP.CGTOther.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
        
        ^TRT475 = TRT:RP:BusinessDetails:Income:Capital.Losses.CarriedForward.Net.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432213(CGTS2018 childDocument)
        {
            bool assertion = (childDocument.CGTS124.GetValueOrDefault() + childDocument.CGTS125.GetValueOrDefault()) != parentDocument.TRT475.GetValueOrDefault();
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.434088",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital losses carried forward to later income years must equal amount in CGT schedule",
                    LongDescription = @"The total of the 'Unapplied net capital losses carried forward'  in the Capital Gains Tax schedule must equal 'Net capital losses carried forward to later income years' on the main form",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:CapitalLossesCarriedForwardNetA",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432213" },
                        new ProcessMessageParameter() { Name = "CGTS124", Value = childDocument.CGTS124.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "CGTS125", Value = childDocument.CGTS125.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "TRT475", Value = parentDocument.TRT475.GetValueOrDefault().ToString() },
                    },
                });
            }
        }
        #endregion // VR.ATO.TRT.432213

        #region VR.ATO.TRT.432042

        /*  VR.ATO.TRT.432042
        If Net capital gain is greater than $10,000, a Capital Gains Tax schedule must be present
         
        Legacy Rule Format:
        ^TRT133 > 10000 AND CountDocument('CGTS') = 0
         
        Technical Business Rule Format:
        ^TRT133 > 10000 AND CountDocument('CGTS') = 0
         
        Data Elements:

        ^TRT133 = TRT:RP:BusinessDetails:Income:CapitalGains:Income.CapitalGainsNet.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432042()
        {
            bool assertion = parentDocument.TRT133.GetValueOrDefault() > 10000 && !cgtsChildDocuments.Any();

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.432042",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital Gains Tax schedule must be present",
                    LongDescription = @"If Net capital gain is greater than $10,000, a Capital Gains Tax schedule must be present",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:OrganisationDetailsTaxConsolidationStatusI",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432042" },
                        new ProcessMessageParameter() { Name = "TRT133", Value = parentDocument.TRT133.GetValueOrDefault().ToString() },
                    },
                });
            }
        }
        #endregion // VR.ATO.TRT.432042

        #region VR.ATO.TRT.433043

        /*  VR.ATO.TRT.433043
        If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business active asset reduction (Subdivision 152-C)", then Small business active asset reduction amount on attached CGT schedule must be completed.

        Legacy Rule Format:
        ^TRT448 = 'A - Small business active asset reduction' AND CountDocument('CGTS') = 1 AND ^CGTS127 = NULL

        Technical Business Rule Format:
        ^TRT448 = 'A - Small business active asset reduction' AND CountDocument('CGTS') = 1 AND ^CGTS127 = NULL

        Data Elements:

        ^TRT448 = TRT:RP:BusinessDetails:Income:CapitalGains:CapitalGainsTax.ExemptionOrRolloverApplied.Code

        ^CGTS127 = CGTS:RP:TaxConcession.CapitalGains.SmallBusinessActiveAssetReduction.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT433043(CGTS2018 childDocument)
        {
            bool assertion = (parentDocument.TRT448Collection != null && parentDocument.TRT448Collection.Any(c => c == @"A - Small business active asset reduction")) && childDocument.CGTS127 == null;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438022",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business active asset reduction amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business active asset reduction (Subdivision 152-C)"", then Small business active asset reduction amount on attached CGT schedule must be completed",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:CapitalGains/tns:TaxExemptionOrRolloverAppliedCCollection/tns:TaxExemptionOrRolloverAppliedC",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.433043" },
                        new ProcessMessageParameter() { Name = "TRT448", Value = "A - Small business active asset reduction" },
                        new ProcessMessageParameter() { Name = "CGTS127", Value = "null" }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.433043

        #region VR.ATO.TRT.433044

        /*  VR.ATO.TRT.433044
        If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business retirement exemption (Subdivision152-D)", then Small business retirement exemption amount on attached CGT schedule must be completed.

        Legacy Rule Format:
        ^TRT448 = 'B - Small business retirement exemption' AND CountDocument('CGTS') = 1 AND ^CGTS128 = NULL

        Technical Business Rule Format:
        ^TRT448 = 'B - Small business retirement exemption' AND CountDocument('CGTS') = 1 AND ^CGTS128 = NULL

        Data Elements:

        ^TRT448 = TRT:RP:BusinessDetails:Income:CapitalGains:CapitalGainsTax.ExemptionOrRolloverApplied.Code

        ^CGTS128 = CGTS:RP:TaxConcession.CapitalGains.SmallBusinessRetirementExemption.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT433044(CGTS2018 childDocument)
        {
            bool assertion = (parentDocument.TRT448Collection != null && parentDocument.TRT448Collection.Any(c => c == @"B - Small business retirement exemption")) && childDocument.CGTS128 == null;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business retirement exemption amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business retirement exemption (Subdivision152-D)"", then Small business retirement exemption amount on attached CGT schedule must be completed",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:CapitalGains/tns:TaxExemptionOrRolloverAppliedCCollection/tns:TaxExemptionOrRolloverAppliedC",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.433044" },
                        new ProcessMessageParameter() { Name = "TRT448", Value = "B - Small business retirement exemption" },
                        new ProcessMessageParameter() { Name = "CGTS128", Value = "null" }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.433044

        #region VR.ATO.TRT.433045

        /*  VR.ATO.TRT.433045
        If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business roll-over (Subdivision 152-E)", then Small business roll-over amount on attached CGT schedule must be completed.

        Legacy Rule Format:
        ^TRT448 = 'C - Small business roll-over' AND CountDocument('CGTS') = 1 AND ^CGTS129 = NULL

        Technical Business Rule Format:
        ^TRT448 = 'C - Small business roll-over' AND CountDocument('CGTS') = 1 AND ^CGTS129 = NULL

        Data Elements:
    
        ^TRT448 = TRT:RP:BusinessDetails:Income:CapitalGains:CapitalGainsTax.ExemptionOrRolloverApplied.Code

        ^CGTS129 = CGTS:RP:TaxConcession.CapitalGains.SmallBusinessRollover.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT433045(CGTS2018 childDocument)
        {
            bool assertion = (parentDocument.TRT448Collection != null && parentDocument.TRT448Collection.Any(c => c == @"C - Small business roll-over")) && childDocument.CGTS129 == null;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438024",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business roll-over amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business roll-over (Subdivision 152-E)"", then Small business roll-over amount on attached CGT schedule must be completed",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:CapitalGains/tns:TaxExemptionOrRolloverAppliedCCollection/tns:TaxExemptionOrRolloverAppliedC",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.433045" },
                        new ProcessMessageParameter() { Name = "TRT448", Value = "C - Small business roll-over" },
                        new ProcessMessageParameter() { Name = "CGTS129", Value = "null" }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.433045

        #region VR.ATO.TRT.433046

        /*  VR.ATO.TRT.433046
        If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business 15 year exemption (Subdivision152-B)", then Small business 15 year exemption amount on attached CGT schedule must be completed.

        Legacy Rule Format:
        ^TRT448 = 'D - Small business 15 year exemption' AND CountDocument('CGTS') = 1 AND ^CGTS138 = NULL

        Technical Business Rule Format:
        ^TRT448 = 'D - Small business 15 year exemption' AND CountDocument('CGTS') = 1 AND ^CGTS138 = NULL

        Data Elements:

        ^TRT448 = TRT:RP:BusinessDetails:Income:CapitalGains:CapitalGainsTax.ExemptionOrRolloverApplied.Code

        ^CGTS138 = CGTS:RP:TaxConcession.CapitalGains.SmallBusinessExemptionExempt.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT433046(CGTS2018 childDocument)
        {
            bool assertion = (parentDocument.TRT448Collection != null && parentDocument.TRT448Collection.Any(c => c == @"D - Small business 15 year exemption")) && childDocument.CGTS138 == null;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438025",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business 15 year exemption amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business 15 year exemption (Subdivision152-B)"", then Small business 15 year exemption amount on attached CGT schedule must be completed",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:CapitalGains/tns:TaxExemptionOrRolloverAppliedCCollection/tns:TaxExemptionOrRolloverAppliedC",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.433046" },
                        new ProcessMessageParameter() { Name = "TRT448", Value = "D - Small business 15 year exemption" },
                        new ProcessMessageParameter() { Name = "CGTS138", Value = "null" }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.433046

        #region VR.ATO.TRT.433047

        /*  VR.ATO.TRT.433047
        If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Foreign resident CGT exemption (Division 855)", then Capital gains disregarded by a foreign resident amount on attached CGT schedule must be completed.

        Legacy Rule Format:
        ^TRT448 = 'E - Foreign resident CGT exemption' AND CountDocument('CGTS') = 1 AND ^CGTS140 = NULL

        Technical Business Rule Format:
        ^TRT448 = 'E - Foreign resident CGT exemption' AND CountDocument('CGTS') = 1 AND ^CGTS140 = NULL

        Data Elements: 

        ^TRT448 = TRT:RP:BusinessDetails:Income:CapitalGains:CapitalGainsTax.ExemptionOrRolloverApplied.Code

        ^CGTS140 = CGTS:RP:Capital.Gains.ForeignResidentExemption.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT433047(CGTS2018 childDocument)
        {
            bool assertion = (parentDocument.TRT448Collection != null && parentDocument.TRT448Collection.Any(c => c == @"E - Foreign resident CGT exemption")) && childDocument.CGTS140 == null;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438026",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital gains disregarded by a foreign resident amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Foreign resident CGT exemption (Division 855)"", then Capital gains disregarded by a foreign resident amount on attached CGT schedule must be completed",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:CapitalGains/tns:TaxExemptionOrRolloverAppliedCCollection/tns:TaxExemptionOrRolloverAppliedC",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.433047" },
                        new ProcessMessageParameter() { Name = "TRT448", Value = "E - Foreign resident CGT exemption" },
                        new ProcessMessageParameter() { Name = "CGTS140", Value = "null" }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.433047

        #region VR.ATO.TRT.433048

        /*  VR.ATO.TRT.433048
        If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Scrip for scrip roll-over (Subdivision 124-M)", then Capital gains disregarded as a result of a scrip for scrip roll-over amount on attached CGT schedule must be completed.

        Legacy Rule Format:
        ^TRT448 = 'F - Scrip for scrip roll-over' AND CountDocument('CGTS') = 1 AND ^CGTS45 = NULL

        Technical Business Rule Format:
        ^TRT448 = 'F - Scrip for scrip roll-over' AND CountDocument('CGTS') = 1 AND ^CGTS45 = NULL

        Data Elements:

        ^TRT448 = TRT:RP:BusinessDetails:Income:CapitalGains:CapitalGainsTax.ExemptionOrRolloverApplied.Code

        ^CGTS45 = CGTS:RP:Capital.Gains.ScriptForScriptRolloverDeferral.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT433048(CGTS2018 childDocument)
        {
            bool assertion = (parentDocument.TRT448Collection != null && parentDocument.TRT448Collection.Any(c => c == @"F - Scrip for scrip roll-over")) && childDocument.CGTS45 == null;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438027",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital gains disregarded as a result of a scrip for scrip roll-over amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Scrip for scrip roll-over (Subdivision 124-M)"", then Capital gains disregarded as a result of a scrip for scrip roll-over amount on attached CGT schedule must be completed",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:CapitalGains/tns:TaxExemptionOrRolloverAppliedCCollection/tns:TaxExemptionOrRolloverAppliedC",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.433048" },
                        new ProcessMessageParameter() { Name = "TRT448", Value = "F - Scrip for scrip roll-over" },
                        new ProcessMessageParameter() { Name = "CGTS45", Value = "null" }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.433048

        #region VR.ATO.TRT.433049

        /*  VR.ATO.TRT.433049
        If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Demerger exemption (Subdivision 125-C)", then Capital gains disregarded by a demerging entity amount on attached CGT schedule must be completed.

        Legacy Rule Format:
        ^TRT448 = 'H - Demerger exemption' AND CountDocument('CGTS') = 1 AND ^CGTS47 = NULL

        Technical Business Rule Format:
        ^TRT448 = 'H - Demerger exemption' AND CountDocument('CGTS') = 1 AND ^CGTS47 = NULL

        Data Elements:

        ^TRT448 = TRT:RP:BusinessDetails:Income:CapitalGains:CapitalGainsTax.ExemptionOrRolloverApplied.Code

        ^CGTS47 = CGTS:RP:Capital.Gains.DemergingEntityExemption.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT433049(CGTS2018 childDocument)
        {
            bool assertion = (parentDocument.TRT448Collection != null && parentDocument.TRT448Collection.Any(c => c == @"H - Demerger exemption")) && childDocument.CGTS47 == null;
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438028",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital gains disregarded by a demerging entity amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Demerger exemption (Subdivision 125-C)"", then Capital gains disregarded by a demerging entity amount on attached CGT schedule must be completed",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:CapitalGains/tns:TaxExemptionOrRolloverAppliedCCollection/tns:TaxExemptionOrRolloverAppliedC",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.433049" },
                        new ProcessMessageParameter() { Name = "TRT448", Value = "H - Demerger exemption" },
                        new ProcessMessageParameter() { Name = "CGTS47", Value = "null" }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.433049

        #endregion  CGTS CrossForm Validation Rules

        #region DISTBENTRT CrossForm Validation Rules

        #region VR.ATO.TRT.442894

        /*  VR.ATO.TRT.442894
        Reporting period start date for the Distributions to Beneficiaries of Trust schedule must be the same as the Reporting period start date of the parent form

        Legacy Rule Format:
        CountDocument('DISTBENTRT') = 1 AND ^TRT573 <> ^DISTBENTRT1

        Technical Business Rule Format:
        CountDocument('DISTBENTRT') = 1 AND ^TRT573 <> ^DISTBENTRT1

        Data Elements:

        ^TRT573 = TRT:RP:Period.Start.Date
        
        ^DISTBENTRT1 = DISTBENTRT:RP:StatementOfDistributionToBeneficiary:Information:Income.DistributionFranked.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public void VRATOTRT442894(DISTBENTRT2024 childDocument)
        {
            bool assertion = (childDocument.DISTBENTRT1 != null) && (childDocument.DISTBENTRT1.GetValueOrDefault() != parentDocument.TRT573.GetValueOrDefault());

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.442894",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Reporting period start date for the Distributions to Beneficiaries of Trust schedule is invalid",
                    LongDescription = @"Reporting period start date for the Distributions to Beneficiaries of Trust schedule must be the same as the Reporting period start date of the parent form",
                    Location = "/tns:TRT/tns:RP/tns:PeriodStartD",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRT.442894" } ,
                        new ProcessMessageParameter { Name = "TRT573", Value = parentDocument.TRT573.HasValue ? parentDocument.TRT573.Value.ToString("yyyy-MM-dd"):"Null" },
                        new ProcessMessageParameter { Name = "DISTBENTRT1", Value = childDocument.DISTBENTRT1.HasValue ?  childDocument.DISTBENTRT1.Value.ToString("yyyy-MM-dd"):"Null" },
                    }
                });
            }
        }

        #endregion // VR.ATO.TRT.442894

        #region VR.ATO.TRT.442895

        /*  VR.ATO.TRT.442895
        Reporting period end date for the Distributions to Beneficiaries of Trust schedule must be the same as the Reporting period end date of the parent form

        Legacy Rule Format:
        CountDocument('DISTBENTRT') = 1 AND ^TRT574 <> ^DISTBENTRT2

        Technical Business Rule Format:
        CountDocument('DISTBENTRT') = 1 AND ^TRT574 <> ^DISTBENTRT2

        Data Elements:

        ^TRT574 = TRT:RP:Period.End.Date

        ^DISTBENTRT2 = DISTBENTRT:RP:Identifiers.TaxFileNumber.Identifier
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public void VRATOTRT442895(DISTBENTRT2024 childDocument)
        {
            bool assertion = (childDocument.DISTBENTRT2 != null) && (childDocument.DISTBENTRT2.GetValueOrDefault() != parentDocument.TRT574.GetValueOrDefault());

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.442895",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Reporting period end date for the Distributions to Beneficiaries of Trust schedule is invalid",
                    LongDescription = @"Reporting period end date for the Distributions to Beneficiaries of Trust schedule must be the same as the Reporting period end date of the parent form",
                    Location = "/tns:TRT/tns:RP/tns:PeriodEndD",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRT.442895" } ,
                        new ProcessMessageParameter { Name = "TRT574", Value = parentDocument.TRT574.HasValue ? parentDocument.TRT574.Value.ToString("yyyy-MM-dd"):"Null" },
                        new ProcessMessageParameter { Name = "DISTBENTRT1", Value = childDocument.DISTBENTRT2.HasValue ?  childDocument.DISTBENTRT2.Value.ToString("yyyy-MM-dd"):"Null" },
                    }
                });
            }
        }

        #endregion // VR.ATO.TRT.442895

        #region VR.ATO.TRT.442896

        /*  VR.ATO.TRT.442896
        The Tax File Number (TFN) provided for the Distributions to Beneficiaries of Trust schedule must be the same as the TFN of the parent form

        Legacy Rule Format:
        CountDocument('DISTBENTRT') = 1 AND ^TRT5 <> ^DISTBENTRT3

        Technical Business Rule Format:
        CountDocument('DISTBENTRT') = 1 AND ^TRT5 <> ^DISTBENTRT3

        Data Elements:

        ^TRT5 = TRT:RP:Identifiers.TaxFileNumber.Identifier
        
        ^DISTBENTRT3 = DISTBENTRT:RP:Identifiers.TaxFileNumber.Identifier
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public void VRATOTRT442896(DISTBENTRT2024 childDocument)
        {
            bool assertion = (childDocument.DISTBENTRT3 != null) && (childDocument.DISTBENTRT3 != null) && (childDocument.DISTBENTRT3 != parentDocument.TRT5);

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.442896",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax File Number for the Distributions to Beneficiaries of Trust schedule is invalid",
                    LongDescription = @"The Tax File Number (TFN) provided for the Distributions to Beneficiaries of Trust schedule must be the same as the TFN of the parent form",
                    Location = "/tns:TRT/tns:RP/tns:TaxFileNumberId",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRT.442896" } ,
                        new ProcessMessageParameter { Name = "TRT5", Value = parentDocument.TRT574.HasValue ? parentDocument.TRT5.ToString():"Null" },
                        new ProcessMessageParameter { Name = "DISTBENTRT3", Value = childDocument.DISTBENTRT3.ToString() },
                    }
                });
            }
        }

        #endregion // VR.ATO.TRT.442896

        #region VR.ATO.TRT.442897

        /*  VR.ATO.TRT.442897
        Distributions to Beneficiaries of Trust schedule must be present when Trust distributions - Income amounts or credits are provided except where type of trust is CCIV Sub-Fund Trust or Is the trust a managed investment trust (MIT)? is yes (true)

        Legacy Rule Format:
        CountDocument('DISTBENTRT') = 0 AND ^TRT32 <> '233' AND ^TRT520 <> TRUE AND (^TRT77> 0 OR ^TRT81> 0 OR ^TRT437 > 0)

        Technical Business Rule Format:
        CountDocument('DISTBENTRT') = 0 AND ^TRT32 <> '233' AND ^TRT520 <> TRUE AND (^TRT77> 0 OR ^TRT81> 0 OR ^TRT437 > 0)

        Data Elements:

        ^TRT77 = TRT:RP:BusinessDetails:Income:PartnershipsTrustsPrimaryProduction:Income.TrustShareNet.Amount

        ^TRT81 = TRT:RP:BusinessDetails:Income:PartnershipTrustNonPrimaryProduction:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount

        ^TRT437 = TRT:RP:BusinessDetails:Income:PartnershipTrustNonPrimaryProduction:Income.TrustDistributionFranked.Amount

        ^TRT520 = TRT:RP:TypeOfTrust:OrganisationDetails.ManagedInvestmentTrust.Indicator

        ^TRT32 = TRT:RP:TypeOfTrust:OrganisationDetails.OrganisationType.Code
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public void VRATOTRT442897()
        {
            bool assertion = ((parentDocument.TRT77.HasValue && parentDocument.TRT77.GetValueOrDefault() > 0) || (parentDocument.TRT81.HasValue && parentDocument.TRT81.GetValueOrDefault() > 0)
                    || (parentDocument.TRT437.HasValue && parentDocument.TRT437.GetValueOrDefault() > 0)) && (distbentrtChildDocuments.Count() == 0) && (!parentDocument.TRT520.HasValue || parentDocument.TRT520 != true) && (string.IsNullOrEmpty(parentDocument.TRT32) || !parentDocument.TRT32.Equals("233"));

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.442897",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Distributions to Beneficiaries of Trust schedule must be present",
                    LongDescription = @"Distributions to Beneficiaries of Trust schedule must be present when Trust distributions - Income amounts or credits are provided except where type of trust is CCIV Sub-Fund Trust or Is the trust a managed investment trust (MIT)? is yes (true)",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:PartnershipsTrustsPrimaryProduction/tns:TrustShareNetA",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.442897" },
                        new ProcessMessageParameter { Name = "TRT77", Value = parentDocument.TRT77.HasValue ? parentDocument.TRT77.GetValueOrDefault().ToString(): "null" },
                        new ProcessMessageParameter { Name = "TRT81", Value = parentDocument.TRT81.HasValue ? parentDocument.TRT81.GetValueOrDefault().ToString(): "null" },
                        new ProcessMessageParameter { Name = "TRT437", Value = parentDocument.TRT437.HasValue ? parentDocument.TRT437.GetValueOrDefault().ToString(): "null"},
                        new ProcessMessageParameter { Name = "TRT520", Value = parentDocument.TRT520.HasValue ? parentDocument.TRT520.GetValueOrDefault().ToString().ToLower(): "null" },
                        new ProcessMessageParameter { Name = "TRT32", Value =  ! string.IsNullOrWhiteSpace(parentDocument.TRT32)? parentDocument.TRT32.ToString(): "null"}
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.442897

        #region VR.ATO.TRT.W00021

        /*  VR.ATO.TRT.W00021
        Distributions to Beneficiaries of Trust schedule is provided but Trust distributions - Income amounts or credits are not declared

        Legacy Rule Format:
        CountDocument('DISTBENTRT') = 1 AND ^TRT77 = NULL AND ^TRT81 = NULL AND ^TRT437 = NULL AND ^TRT85 = NULL

        Technical Business Rule Format:
        CountDocument('DISTBENTRT') = 1 AND ^TRT77 = NULL AND ^TRT81 = NULL AND ^TRT437 = NULL AND ^TRT85 = NULL

        Data Elements:
    
        ^TRT77 = TRT:RP:BusinessDetails:Income:PartnershipsTrustsPrimaryProduction:Income.TrustShareNet.Amount
    
        ^TRT81 = TRT:RP:BusinessDetails:Income:PartnershipTrustNonPrimaryProduction:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
    
        ^TRT85 = TRT:RP:BusinessDetails:Income:ShareOfCredits:IncomeTax.FrankingCredits.FrankingCreditShareReceivedIndirectly.Amount
    
        ^TRT437 = TRT:RP:BusinessDetails:Income:PartnershipTrustNonPrimaryProduction:Income.TrustDistributionFranked.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public void VRATOTRTW00021(DISTBENTRT2024 childDocument)
        {
            bool assertion = (!parentDocument.TRT77.HasValue) && (!parentDocument.TRT81.HasValue) && (!parentDocument.TRT85.HasValue) && (!parentDocument.TRT437.HasValue);
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.W00021",
                    Severity = ProcessMessageSeverity.Warning,
                    Description = @"Ensure Trust distributions - Income amounts or credits from trusts income are declared",
                    LongDescription = @"Ensure Trust distributions - Income amounts or credits are declared when Distributions to Beneficiaries of Trust schedule is provided",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:PartnershipsTrustsPrimaryProduction/tns:TrustShareNetA",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRT.W00021" },
                        new ProcessMessageParameter { Name = "TRT77", Value = parentDocument.TRT77.HasValue ? parentDocument.TRT77.GetValueOrDefault().ToString(): "null" },
                        new ProcessMessageParameter { Name = "TRT81", Value = parentDocument.TRT81.HasValue ? parentDocument.TRT81.GetValueOrDefault().ToString(): "null" },
                        new ProcessMessageParameter { Name = "TRT85", Value = parentDocument.TRT85.HasValue ? parentDocument.TRT85.GetValueOrDefault().ToString(): "null" },
                        new ProcessMessageParameter { Name = "TRT437", Value = parentDocument.TRT437.HasValue ? parentDocument.TRT437.GetValueOrDefault().ToString(): "null"}
                    }
                });
            }
        }

        #endregion // VR.ATO.TRT.W00021

        #endregion // DISTBENTRT CrossForm Validation Rules

        #region FTER CrossForm Validation Rules

        #region VR.ATO.TRT.432017

        /*  VR.ATO.TRT.432017
        Family Trust Election status code selected requires a Family trust election, revocation or variation schedule to be completed and attached to return and there is no schedule present.

        Legacy Rule Format:
        ^TRT29 <> NULL AND CountDocument('FTER') = 0

        Technical Business Rule Format:
        ^TRT29 <> NULL AND CountDocument('FTER') = 0

        Data Elements:
    
        ^TRT29 = TRT:RP:ElectionStatus:Elections.FamilyTrustElectionRevocation.Code
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432017()
        {
            bool assertion = parentDocument.TRT29 != null && !fterChildDocuments.Any();
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.432017",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Family trust election or revocation schedule must be present",
                    Location = "/tns:TRT/tns:RP/tns:ElectionStatus/tns:ElectionsFamilyTrustElectionRevocationC",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432017" },
                        new ProcessMessageParameter() { Name = "TRT29", Value = parentDocument.TRT29 }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.432017

        #endregion  //FTER CrossForm Validation Rules

        #region IDS CrossForm Validation Rules

        #region VR.ATO.TRT.432646

        /*  VR.ATO.TRT.432646
        The sum of the amounts showing at attributable income of controlled foreign companies and controlled foreign trusts on IDS should be equal to or less than the sum of the amounts at Listed country and Unlisted country on the main return form.

        Legacy Rule Format:
        CountDocument('IDS') = 1 AND (^IDS191 + ^IDS192 + ^IDS193) > (^TRT134 + ^TRT136)

        Technical Business Rule Format:
        CountDocument('IDS') = 1 AND (^IDS191 + ^IDS192 + ^IDS193) > (^TRT134 + ^TRT136)

        Data Elements:
    
        ^TRT134 = TRT:RP:BusinessDetails:Income:AttributedForeignIncome:ListedCountry:Income.InternationalDealings.Attributed.Amount

        ^TRT136 = TRT:RP:BusinessDetails:Income:AttributedForeignIncome:UnlistedCountry:Income.InternationalDealings.Attributed.Amount

        ^IDS191 = IDS:RP:Income.ControlledForeignCompaniesAttributableIncomeTotal.Amount
    
        ^IDS192 = IDS:RP:Income.ControlledForeignCompaniesChangeOfResidenceIncome.Amount
    
        ^IDS193 = IDS:RP:Income.ControlledForeignCompaniesAndControlledForeignTrustsInterposedAustralianEntitiesAttributableIncomeTotal.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432646(IDS2025 childDocument)
        {
            decimal totalIDS = childDocument.IDS191.GetValueOrDefault() + childDocument.IDS192.GetValueOrDefault() + childDocument.IDS193.GetValueOrDefault();
            decimal totalTRT = parentDocument.TRT134.GetValueOrDefault() + parentDocument.TRT136.GetValueOrDefault();

            bool assertion = totalIDS > totalTRT;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.432714",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Attributed foreign income incorrect",
                    LongDescription = @"The sum of amounts at attributable income of controlled foreign companies and controlled foreign trusts on the IDS should be equal to or less than the sum of amounts at Listed country and Unlisted country on the trust return form.",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:AttributedForeignIncome/tns:UnlistedCountryInternationalDealingsA",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432646" },
                        new ProcessMessageParameter() { Name = "TRT134", Value = parentDocument.TRT134.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "TRT136", Value = parentDocument.TRT136.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "Total TRT fields", Value = totalTRT.ToString() },
                        new ProcessMessageParameter() { Name = "IDS191", Value = childDocument.IDS191.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "IDS192", Value = childDocument.IDS192.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "IDS193", Value = childDocument.IDS193.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "Total IDS fields", Value = totalIDS.ToString() },
                    },
                });
            }
        }
        #endregion // VR.ATO.TRT.432646

        #region VR.ATO.TRT.432647

        /*  VR.ATO.TRT.432647
        The Section 456 Controlled Foreign Companies attributable income (Listed countries) amount on the International Dealings Schedule must not exceed the Attributed foreign income (Listed country) amount on the parent Trust return

        Legacy Rule Format:
        CountDocument('IDS') = 1 AND ^TRT134 > 0 AND ^IDS188 > ^TRT134

        Technical Business Rule Format:
        CountDocument('IDS') = 1 AND ^TRT134 > 0 AND ^IDS188 > ^TRT134

        Data Elements:
    
        ^TRT134 = TRT:RP:BusinessDetails:Income:AttributedForeignIncome:ListedCountry:Income.InternationalDealings.Attributed.Amount
    
        ^IDS188 = IDS:RP:ListedCFCAmounts:Income.ControlledForeignCompaniesAttributableIncome.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432647(IDS2025 childDocument)
        {
            bool assertion = parentDocument.TRT134.GetValueOrDefault() > 0 && childDocument.IDS188.GetValueOrDefault() > parentDocument.TRT134.GetValueOrDefault();

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.442647",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Attributed foreign income incorrect",
                    LongDescription = "The Section 456 Controlled Foreign Companies attributable income (Listed countries) amount on the International Dealings Schedule must not exceed the Attributed foreign income (Listed country) amount on the parent Trust return",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:AttributedForeignIncome/tns:ListedCountryInternationalDealingsA",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432647" },
                        new ProcessMessageParameter() { Name = "TRT134", Value = parentDocument.TRT134.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "IDS188", Value = childDocument.IDS188.GetValueOrDefault().ToString() },
                    },
                });
            }
        }
        #endregion // VR.ATO.TRT.432647

        #region VR.ATO.TRT.432648

        /*  VR.ATO.TRT.432648
        Where a positive amount is present at Attributed foreign income (Listed country or Unlisted country) on the Trust return, the Number of CFCs and CFTs in which you had an interest at the end of your income year (Listed countries, Specified countries or Other unlisted countries) must be provided on the International Dealings Schedule. If at the end of your income year you no longer held an interest in CFCs or CFTs, enter '0' (zero).

        Legacy Rule Format:
        (^TRT134 > 0 OR ^TRT136 > 0) AND (CountDocument('IDS') = 0 OR (^IDS185 = NULL AND ^IDS186 = NULL AND ^IDS187 = NULL))

        Technical Business Rule Format:
        (^TRT134 > 0 OR ^TRT136 > 0) AND (CountDocument('IDS') = 0 OR (^IDS185 = NULL AND ^IDS186 = NULL AND ^IDS187 = NULL))

        Data Elements:
    
        ^TRT134 = TRT:RP:BusinessDetails:Income:AttributedForeignIncome:ListedCountry:Income.InternationalDealings.Attributed.Amount

        ^TRT136 = TRT:RP:BusinessDetails:Income:AttributedForeignIncome:UnlistedCountry:Income.InternationalDealings.Attributed.Amount
    
        ^IDS185 = IDS:RP:Listed: Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count
    
        ^IDS186 = IDS:RP:Specified: Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count
    
        ^IDS187 = IDS:RP:Unlisted: Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432648()
        {
            bool assertion = (parentDocument.TRT134.GetValueOrDefault() > 0 || parentDocument.TRT136.GetValueOrDefault() > 0) && (!idsChildDocuments.Any() || idsChildDocuments.Any(ids => ids.IDS185 == null && ids.IDS186 == null && ids.IDS187 == null));

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.442648",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of controlled foreign companies or trusts required",
                    LongDescription = @"Where a positive amount is present at Attributed foreign income (Listed country or Unlisted country) on the Trust return, the Number of CFCs and CFTs in which you had an interest at the end of your income year (Listed countries, Specified countries or Other unlisted countries) must be provided on the International Dealings Schedule. If at the end of your income year you no longer held an interest in CFCs or CFTs, enter '0' (zero).",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:AttributedForeignIncome/tns:ListedCountryInternationalDealingsA",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432648" },
                        new ProcessMessageParameter() { Name = "TRT134", Value = parentDocument.TRT134.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "TRT136", Value = parentDocument.TRT136.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "IDS185", Value = "null" },
                        new ProcessMessageParameter() { Name = "IDS186", Value = "null" },
                        new ProcessMessageParameter() { Name = "IDS187", Value = "null" }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.432648

        #region VR.ATO.TRT.432649

        /*  VR.ATO.TRT.432649
        If there is a yes response to question 4 label A on the IDS, there must also be a yes response to question 29 label C (transactions with specified countries) on the trust or partnership return form.

        Legacy Rule Format:
        ^TRT151 = FALSE AND CountDocument('IDS') = 1 AND ^IDS9 = TRUE

        Technical Business Rule Format:
        ^TRT151 = FALSE AND CountDocument('IDS') = 1 AND ^IDS9 = TRUE

        Data Elements:
    
        ^TRT151 = TRT:RP:BusinessDetails:Income:AttributedForeignIncome:Income:ThinCapitalisation.InternationalDealingsTransactionsWithTaxHavenCountriesI
    
        ^IDS9 = IDS:RP:InternationalDealings.SpecifiedCountriesTransactions.Indicator
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432649(IDS2025 childDocument)
        {
            bool assertion = parentDocument.TRT151.GetValueOrDefault() == false && childDocument.IDS9.GetValueOrDefault() == true;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.432649",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Transactions with specified countries question incorrect",
                    LongDescription = "If there is a 'True' response at 'Did you have dealings with international related parties located in specified countries?' on the IDS then the response at Overseas transactions - Transactions with specified countries on the trust return must be 'True'.",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:ThinCapitalisation/tns:InternationalDealingsTransactionsWithTaxHavenCountriesI",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432649" },
                        new ProcessMessageParameter() { Name = "TRT151", Value = parentDocument.TRT151.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "IDS9", Value = childDocument.IDS9.GetValueOrDefault().ToString() },
                    },
                });
            }
        }
        #endregion // VR.ATO.TRT.432649

        #region VR.ATO.TRT.432650

        /*  VR.ATO.TRT.432650
        If Debt deductions in earning non-assessable foreign income claimed under s25-90 has been completed on the IDS then Total interest expenses or Other deductions or Interest expenses overseas must be greater than zero on trust return form.

        Legacy Rule Format:
        CountDocument('IDS') = 1 AND ^IDS207 > 0 AND (^TRT470 = NULL OR ^TRT470 = 0) AND (^TRT182 = NULL OR ^TRT182 = 0) AND (^TRT129 = NULL OR ^TRT129 = 0)

        Technical Business Rule Format:
        CountDocument('IDS') = 1 AND ^IDS207 > 0 AND (^TRT470 = NULL OR ^TRT470 = 0) AND (^TRT182 = NULL OR ^TRT182 = 0) AND (^TRT129 = NULL OR ^TRT129 = 0)

        Data Elements:
    
        ^TRT129 = TRT:RP:BusinessDetails:Income:Deductions:OtherDeductions:Expense.DeductibleOther.Amount
    
        ^TRT182 = TRT:RP:BusinessDetails:Income:ThinCapitalisation:Expense.Interest.Amount
        
        ^TRT470 = TRT:RP:BusinessDetails:Income:BusinessIncomeAndExpenses:BusinessExpenses:Expense.Operating.Interest.Amount
        
        ^IDS207 = IDS:RP:IncomeTax.Deduction.DebtEarningNonAssessableNonExemptForeignIncomeTotal.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432650(IDS2025 childDocument)
        {
            bool assertion = childDocument.IDS207.GetValueOrDefault() > 0 && (parentDocument.TRT470 == null || parentDocument.TRT470 == 0) && (parentDocument.TRT182 == null || parentDocument.TRT182 == 0) && (parentDocument.RP_BusinessDetails_Income_Deductions_OtherDeductionsCollection != null && parentDocument.RP_BusinessDetails_Income_Deductions_OtherDeductionsCollection.Any(tuple => tuple.TRT129 == null || tuple.TRT129 == 0));

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.432650",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Interest expenses required",
                    LongDescription = @"If Debt deductions in earning non-assessable foreign income claimed under s25-90 has been completed on the IDS then Total interest expenses or Other deductions or Interest expenses overseas must be greater than zero on trust return form.",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:BusinessIncomeAndExpenses/tns:BusinessExpenses/tns:OperatingInterestA",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432650" },
                        new ProcessMessageParameter() { Name = "TRT129", Value = parentDocument.RP_BusinessDetails_Income_Deductions_OtherDeductionsCollection.Find(tuple => tuple.TRT129 == null || tuple.TRT129 == 0).TRT129.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "TRT182", Value = parentDocument.TRT182.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "TRT470", Value = parentDocument.TRT470.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "IDS207", Value = childDocument.IDS207.GetValueOrDefault().ToString() }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.432650

        #region VR.ATO.TRT.432651

        /*  VR.ATO.TRT.432651
        Where there is a positive amount present at 'Total amount of your share of FHLP/FHCs net income/profit' on the International Dealings Schedule, the 'Gross amount of Other assessable foreign source income, excluding income shown at Attributed foreign income' amount on the Trust Return must be provided

        Legacy Rule Format:
        CountDocument('IDS') = 1 AND ^TRT140 = NULL AND ^IDS219 > 0

        Technical Business Rule Format:
        CountDocument('IDS') = 1 AND ^TRT140 = NULL AND ^IDS219 > 0

        Data Elements:
    
        ^TRT140 = TRT:RP:BusinessDetails:Income:AttributedForeignIncome:OtherAssessableForeignIncome:Income.Other.Amount
    
        ^IDS219 = IDS:RP:Assets.Investment.ForeignHybridLimitedPartnershipOrForeignHybridCompanyNetIncomeProfitTotal.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432651(IDS2025 childDocument)
        {
            bool assertion = parentDocument.TRT140 == null && childDocument.IDS219.GetValueOrDefault() > 0;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.442651",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross other assessable foreign source income required",
                    LongDescription = @"Where there is a positive amount present at 'Total amount of your share of FHLP/FHCs net income/profit' on the International Dealings Schedule, the 'Gross amount of Other assessable foreign source income, excluding income shown at Attributed foreign income' amount on the Trust Return must be provided",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:AttributedForeignIncome/tns:OtherAssessableForeignIncome/tns:A",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432651" },
                        new ProcessMessageParameter() { Name = "TRT140", Value = parentDocument.TRT140.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "IDS219", Value = childDocument.IDS219.GetValueOrDefault().ToString() },
                    },
                });
            }
        }
        #endregion // VR.ATO.TRT.432651

        #region VR.ATO.TRT.432652

        /*  VR.ATO.TRT.432652
        If there is a 'True' response to 'Did you have any capital gains tax (CGT) events in relation to your interest in a foreign company?' On the IDS then a 'True' response is required at 'Did you have a CGT event during the year?' on the trust return.

        Legacy Rule Format:
        ^TRT131 = FALSE AND ^IDS208 = TRUE

        Technical Business Rule Format:
        ^TRT131 = FALSE AND ^IDS208 = TRUE

        Data Elements:
    
        ^TRT131 = TRT:RP:BusinessDetails:Income:CapitalGains:CapitalGainsTax.Event.Indicator
    
        ^IDS208 = IDS:RP:CapitalGainsTax.Event.Indicator
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432652(IDS2025 childDocument)
        {
            bool assertion = parentDocument.TRT131 == false && childDocument.IDS208 == true;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.432652",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Check response to CGT event question",
                    LongDescription = @"If there is a 'True' response to 'Did you have any capital gains tax (CGT) events in relation to your interest in a foreign company?' On the IDS then a 'True' response is required at 'Did you have a CGT event during the year?' on the trust return.",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:CapitalGains/tns:TaxEventI",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432652" },
                        new ProcessMessageParameter() { Name = "TRT131", Value = parentDocument.TRT131.ToString() },
                        new ProcessMessageParameter() { Name = "IDS208", Value = childDocument.IDS208.ToString() },
                    },
                });
            }
        }
        #endregion // VR.ATO.TRT.432652

        #region VR.ATO.TRT.432653

        /*  VR.ATO.TRT.432653
        If Capital gain amounts has been completed on the IDS then Net capital gain must be completed on the trust return.

        Legacy Rule Format:
        ^TRT133 = NULL AND ^IDS209 > 0

        Technical Business Rule Format:
        ^TRT133 = NULL AND ^IDS209 > 0

        Data Elements:
    
        ^TRT133 = TRT:RP:BusinessDetails:Income:CapitalGains:Income.CapitalGainsNet.Amount
    
        ^IDS209 = IDS:RP:CGTEvents:CapitalGainsTax.ActiveForeignBusinessCapitalGains.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432653(IDS2025 childDocument)
        {
            bool assertion = parentDocument.TRT133 == null && childDocument.IDS209.GetValueOrDefault() > 0;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.432653",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital gain required",
                    LongDescription = @"If Capital gain amounts has been completed on the IDS then Net capital gain must be completed on the trust return.",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:CapitalGains/tns:NetA",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432653" },
                        new ProcessMessageParameter() { Name = "TRT133", Value = parentDocument.TRT133.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "IDS209", Value = childDocument.IDS209.GetValueOrDefault().ToString() },
                    },
                });
            }
        }
        #endregion // VR.ATO.TRT.432653

        #region VR.ATO.TRT.432655

        /*  VR.ATO.TRT.432655
        Where there is a positive Royalties deductions amount present on the International Dealings Schedule, a positive Royalty expenses overseas amount must be provided

        Legacy Rule Format:
        (^TRT183 = 0 OR ^TRT183 = NULL) AND CountDocument('IDS') = 1 AND ^IDS22 > 0

        Technical Business Rule Format:
        (^TRT183 = 0 OR ^TRT183 = NULL) AND CountDocument('IDS') = 1 AND ^IDS22 > 0

        Data Elements:
    
        ^TRT183 = TRT:RP:BusinessDetails:Income:AttributedForeignIncome:Income:ThinCapitalisation.ExpenseRoyaltiesA
    
        ^IDS22 = IDS:RP:Royalties:Expense.DeductionsTotal.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432655(IDS2025 childDocument)
        {
            bool assertion = parentDocument.TRT183.GetValueOrDefault() == 0 && childDocument.IDS22.GetValueOrDefault() > 0;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.442655",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Royalty expenses overseas required",
                    LongDescription = "Where there is a positive Royalties deductions amount present on the International Dealings Schedule, a positive Royalty expenses overseas amount must be provided",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:ThinCapitalisation/tns:ExpenseRoyaltiesA",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432655" },
                        new ProcessMessageParameter() { Name = "TRT183", Value = parentDocument.TRT183.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "IDS22", Value = childDocument.IDS22.GetValueOrDefault().ToString() },
                    },
                });
            }
        }
        #endregion // VR.ATO.TRT.432655

        #region VR.ATO.TRT.432656

        /*  VR.ATO.TRT.432656
        If Royalties deductions has been completed on the IDS then it must be less than or equal to Royalty expenses overseas on the trust return form

        Legacy Rule Format:
        ^TRT183 > 0 AND CountDocument('IDS') = 1 AND ^TRT183 < ^IDS22

        Technical Business Rule Format:
        ^TRT183 > 0 AND CountDocument('IDS') = 1 AND ^TRT183 < ^IDS22

        Data Elements:
    
        ^TRT183 = TRT:RP:BusinessDetails:Income:AttributedForeignIncome:Income:ThinCapitalisation.ExpenseRoyaltiesA
    
        ^IDS22 = IDS:RP:Royalties:Expense.DeductionsTotal.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432656(IDS2025 childDocument)
        {
            bool assertion = parentDocument.TRT183.GetValueOrDefault() > 0 && parentDocument.TRT183.GetValueOrDefault() < childDocument.IDS22.GetValueOrDefault();

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.432656",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Royalties deductions incorrect",
                    LongDescription = "If Royalties deductions has been completed on the IDS then it must be less than or equal to Royalty expenses overseas on the trust return form",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:ThinCapitalisation/tns:ExpenseRoyaltiesA",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432656" },
                        new ProcessMessageParameter() { Name = "TRT183", Value = parentDocument.TRT183.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "IDS22", Value = childDocument.IDS22.GetValueOrDefault().ToString() },
                    },
                });
            }
        }
        #endregion // VR.ATO.TRT.432656

        #region VR.ATO.TRT.432675

        /*  VR.ATO.TRT.432675
        Ensure an International dealings schedule (IDS) is completed when a value greater than zero is present at label Interest expenses overseas       

        Legacy Rule Format:
        ^TRT182 > 0 AND CountDocument('IDS') = 0

        Technical Business Rule Format:
        ^TRT182 > 0 AND CountDocument('IDS') = 0

        Data Elements:
    
        ^TRT182 = TRT:RP:BusinessDetails:Income:AttributedForeignIncome:Income:ThinCapitalisation.ExpenseInterestA
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432675()
        {
            bool assertion = parentDocument.TRT182 > 0 && !idsChildDocuments.Any();

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.432675",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "International Dealings Schedule not attached",
                    LongDescription = "An International dealings schedule (IDS) is not attached when a value greater than zero is present at label Interest expenses overseas",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:ThinCapitalisation/tns:ExpenseInterestA",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432675" },
                        new ProcessMessageParameter() { Name = "TRT182", Value = parentDocument.TRT182.ToString() },
                    },
                });
            }
        }
        #endregion // VR.ATO.TRT.432675

        #region VR.ATO.TRT.432676

        /*  VR.ATO.TRT.432676
        Ensure an International dealings schedule (IDS) is completed when a value greater than zero is present at label Royalty expenses overseas      

        Legacy Rule Format:
        ^TRT183 > 0 AND CountDocument('IDS') = 0

        Technical Business Rule Format:
        ^TRT183 > 0 AND CountDocument('IDS') = 0

        Data Elements:
    
        ^TRT183 = TRT:RP:BusinessDetails:Income:AttributedForeignIncome:Income:ThinCapitalisation.ExpenseRoyaltiesA
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432676()
        {
            bool assertion = parentDocument.TRT183 > 0 && !idsChildDocuments.Any();

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.432676",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "International Dealings Schedule not attached",
                    LongDescription = "An International dealings schedule (IDS) is not attached when a value greater than zero is present at label Royalty expenses overseas",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:ThinCapitalisation/tns:ExpenseRoyaltiesA",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432676" },
                        new ProcessMessageParameter() { Name = "TRT183", Value = parentDocument.TRT183.ToString() },
                    },
                });
            }
        }
        #endregion // VR.ATO.TRT.432676

        #region VR.ATO.TRT.432707

        /*  VR.ATO.TRT.432707
        If there is a 'NO' response to the questions concerning whether the entity had either a direct or indirect interest in a foreign trust, controlled foreign company or transferor trust and whether the value of dealings with international related parties is greater than $2 million and whether the thin capitalisation provisions apply, and no amounts have been completed at Interest expenses overseas and Royalty expenses overseas, an International Dealings schedule must not be attached.

        Legacy Rule Format:
        ^TRT138 = FALSE AND ^TRT148 = FALSE AND ^TRT449 = FALSE AND (^TRT182 = NULL OR ^TRT182 = 0) AND (^TRT183 = NULL OR ^TRT183 = 0) AND CountDocument('IDS') = 1

        Technical Business Rule Format:
        ^TRT138 = FALSE AND ^TRT148 = FALSE AND ^TRT449 = FALSE AND (^TRT182 = NULL OR ^TRT182 = 0) AND (^TRT183 = NULL OR ^TRT183 = 0) AND CountDocument('IDS') = 1

        Data Elements:
    
        ^TRT138 = TRT:RP:BusinessDetails:Income:AttributedForeignIncome:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator
    
        ^TRT148 = TRT:RP:BusinessDetails:Income:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator
    
        ^TRT182 = TRT:RP:BusinessDetails:Income:ThinCapitalisation:Expense.Interest.Amount
    
        ^TRT183 = TRT:RP:BusinessDetails:Income:ThinCapitalisation:Expense.Royalties.Amount
    
        ^TRT449 = TRT:RP:BusinessDetails:Income:ThinCapitalisation:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432707(IDS2025 childDocument)
        {
            bool assertion = parentDocument.TRT138 == false && parentDocument.TRT148 == false && parentDocument.TRT449 == false && (parentDocument.TRT182 == null || parentDocument.TRT182 == 0) && (parentDocument.TRT183 == null || parentDocument.TRT183 == 0);

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.442901",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule is not required",
                    LongDescription = @"If there is a 'NO' response to the questions concerning whether the entity had either a direct or indirect interest in a foreign trust, controlled foreign company or transferor trust and whether the value of dealings with international related parties is greater than $2 million and whether the thin capitalisation or debt deduction creation rules apply, and no amounts have been completed at Interest expenses overseas and Royalty expenses overseas, an International Dealings schedule must not be attached.",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:AttributedForeignIncome/tns:InternationalDealingsTrustorControlledCompanyorTransferorTrustI",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432707" },
                        new ProcessMessageParameter() { Name = "TRT138", Value = TRT2025Validator.GetValueOrEmpty(parentDocument.TRT138) },
                        new ProcessMessageParameter() { Name = "TRT148", Value = TRT2025Validator.GetValueOrEmpty(parentDocument.TRT148) },
                        new ProcessMessageParameter() { Name = "TRT182", Value = parentDocument.TRT182.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "TRT183", Value = parentDocument.TRT183.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "TRT449", Value = TRT2025Validator.GetValueOrEmpty(parentDocument.TRT449) },
                    },
                });
            }
        }
        #endregion // VR.ATO.TRT.432707

        #region VR.ATO.TRT.432720

        /*  VR.ATO.TRT.432720
        If there is an amount at either 'Total TOFA gains' or 'Total TOFA Losses' that indicates you made a gain or loss from a financial arrangement subject to the TOFA rules, then there must be a 'True' response to 'Are you subject to the Taxation of Financial Arrangements rules contained in Division 230 of the ITAA 1997?' on the IDS

        Legacy Rule Format:
        (^TRT356 > 0 OR ^TRT357 > 0) AND CountDocument('IDS') = 1 AND ^IDS177 <> TRUE

        Technical Business Rule Format:
        (^TRT356 > 0 OR ^TRT357 > 0) AND CountDocument('IDS') = 1 AND ^IDS177 <> TRUE

        Data Elements:

        ^TRT356 = TRT:RP:BusinessDetails:Income:TaxationOfFinancialArrangements.GainsTotalA

        ^TRT357 = TRT:RP:BusinessDetails:Income:TaxationOfFinancialArrangements.ExpenseLossesTotalA
    
        ^IDS177 = IDS:RP:InternationalDealings.TaxationOfFinancialArrangementsRulesApplied.Indicator
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432720(IDS2025 childDocument)
        {
            bool assertion = (parentDocument.TRT356 > 0 || parentDocument.TRT357 > 0) && childDocument.IDS177.GetValueOrDefault() != true;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.432762",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Response to TOFA gains or losses is required",
                    LongDescription = @"If there is an amount at either 'Total TOFA gains' or 'Total TOFA Losses' that indicates you made a gain or loss from a financial arrangement subject to the TOFA rules, then there must be a 'True' response to 'Are you subject to the Taxation of Financial Arrangements rules contained in Division 230 of the ITAA 1997?' on the IDS.",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:TaxationOfFinancialArrangements/tns:GainsTotalA",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432720" },
                        new ProcessMessageParameter() { Name = "TRT356", Value = parentDocument.TRT356.ToString() },
                        new ProcessMessageParameter() { Name = "TRT357", Value = parentDocument.TRT357.ToString() },
                        new ProcessMessageParameter() { Name = "IDS177", Value = childDocument.IDS177.ToString() },
                    },
                });
            }
        }
        #endregion // VR.ATO.TRT.432720

        #region VR.ATO.TRT.432797

        /*  VR.ATO.TRT.432797
        If the entity has self-assessed as a Significant Global Entity (SGE), then 'Are you a small business entity, not a significant global entity and your international related party dealings do not exceed $5 million and 50% of your current year aggregated turnover?' must not be 'TRUE' on the International Dealings Schedule (IDS) form.

        Legacy Rule Format:
        ^TRT487 = TRUE AND CountDocument('IDS') = 1 AND ^IDS425 = TRUE

        Technical Business Rule Format:
        ^TRT487 = TRUE AND CountDocument('IDS') = 1 AND ^IDS425 = TRUE

        Data Elements:
    
        ^TRT487 = TRT:RP:BusinessDetails:OrganisationDetails.SignificantGlobalEntityStatus.Indicator
    
        ^IDS425 = IDS:RP:InternationalDealings.SmallBusinessConcessionsEligibility.Indicator
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432797(IDS2025 childDocument)
        {
            bool assertion = (parentDocument.TRT487 == true && childDocument.IDS425.GetValueOrDefault() == true);

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.442797",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Significant Global Entity question on the International Dealings Schedule (IDS) form must not be 'TRUE'.",
                    LongDescription = @"If the entity has self-assessed as a Significant Global Entity (SGE), then 'Are you a small business entity, not a significant global entity and your international related party dealings do not exceed $5 million and 50% of your current year aggregated turnover?' must not be 'TRUE' on the International Dealings Schedule (IDS) form.",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:OrganisationDetailsSignificantGlobalEntityStatusI",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432797" },
                        new ProcessMessageParameter() { Name = "TRT487", Value = parentDocument.TRT487.ToString() },
                        new ProcessMessageParameter() { Name = "IDS425", Value = childDocument.IDS425.ToString() },
                    },
                });
            }
        }

        #endregion // VR.ATO.TRT.432797

        #region VR.ATO.TRT.433001

        /*  VR.ATO.TRT.433001
        If the response to the question to whether the entity had either a direct or indirect interest in a foreign trust, controlled foreign company or transferor trust (Attributed foreign income) is 'Yes, an International Dealings Schedule must be attached

        Legacy Rule Format:
        ^TRT138 = TRUE AND CountDocument('IDS') = 0

        Technical Business Rule Format:
        ^TRT138 = TRUE AND CountDocument('IDS') = 0

        Data Elements:
    
        ^TRT138 = TRT:RP:BusinessDetails:Income:AttributedForeignIncome:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT433001()
        {
            bool assertion = parentDocument.TRT138 == true && !idsChildDocuments.Any();

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438002",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule is not attached",
                    LongDescription = @"The answer to the question 'Did the entity have either a direct or indirect interest in a foreign trust, controlled foreign company or transferor trust?' is 'Yes', and no International Dealings Schedule is attached.",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:AttributedForeignIncome/tns:InternationalDealingsTrustorControlledCompanyorTransferorTrustI",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.433001" },
                        new ProcessMessageParameter() { Name = "TRT138", Value = "true" }
                    }
                });
            }
        }

        #endregion // VR.ATO.TRT.433001

        #region VR.ATO.TRT.433050

        /*  VR.ATO.TRT.433050
        When you were affected by the Thin capitalisation provisions, the International Dealings Schedule (IDS) must be attached

        Legacy Rule Format:
        ^TRT449 = TRUE AND CountDocument('IDS') = 0

        Technical Business Rule Format:
        ^TRT449 = TRUE AND CountDocument('IDS') = 0

        Data Elements:
    
        ^TRT449 = TRT:RP:BusinessDetails:Income:AttributedForeignIncome:Income:ThinCapitalisation.LiabilitiesProvisionsAppliedI
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT433050()
        {
            bool assertion = parentDocument.TRT449 == true && !idsChildDocuments.Any();

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.442902",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "International Dealings Schedule not attached",
                    LongDescription = "If the thin capitalisation or debt deduction creation rules apply then International Dealings Schedule must be attached",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:ThinCapitalisation/tns:LiabilitiesProvisionsAppliedI",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.433050" },
                        new ProcessMessageParameter() { Name = "TRT449", Value = TRT2025Validator.GetValueOrEmpty(parentDocument.TRT449) },
                    },
                });
            }
        }
        #endregion // VR.ATO.TRT.433050

        #endregion // IDS CrossForm Validation Rules

        #region IEE CrossForm Validation Rules

        #region VR.ATO.TRT.432011

        /*  VR.ATO.TRT.432011
        The income year for which the details of the Interposed Entity Election are being revoked must equal the income year of the main return being lodged.

        Legacy Rule Format:
        CountDocument('IEE') > 0 AND AnyOccurrence(^IEE116, ^IEE116 = 'R' AND ^IEE82 <> ^TRT1)

        Technical Business Rule Format:
        CountDocument('IEE') > 0 AND AnyOccurrence(^IEE116, ^IEE116 = 'R' AND ^IEE82 <> ^TRT1)

        Data Elements:

        ^TRT1 = TRT:RP:TargetFinancialY

        ^IEE82 = IEE:RP:Elections.Revocation.Year

        ^IEE116 = IEE:RP:Elections.InterposedEntityElectionOrRevocation.Code
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432011()
        {
            IEE2021 childDocument = null;

            if (ieeChildDocuments.Count() > 0)
            {
                childDocument = ieeChildDocuments.Any(iee => iee.IEE116 == "R" && iee.IEE82 != parentDocument.TRT1) ? ieeChildDocuments.First(iee => iee.IEE116 == "R" && iee.IEE82 != parentDocument.TRT1) : null;
            }

            bool assertion = childDocument != null;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.430176",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Year of revocation must be the same as the year of the return",
                    LongDescription = @"The income year for which the details of the Interposed Entity Election are being revoked must equal the income year of the main return being lodged.",
                    Location = "/tns:TRT/tns:RP/tns:TargetFinancialY",
                    Parameters = new ProcessMessageParameters {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432011" },
                        new ProcessMessageParameter { Name = "TRT1", Value = parentDocument.TRT1.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter { Name = "IEE82", Value = childDocument.IEE82.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter { Name = "IEE116", Value = "R" }
                    }
                });
            }
        }

        #endregion // VR.ATO.TRT.432011

        #region VR.ATO.TRT.432019

        /*  VR.ATO.TRT.432019
        An IEE schedule is attached; however the IEE specified income year has not been completed on the main form.

        Legacy Rule Format:
        ^TRT30 = NULL AND CountDocument('IEE') > 0

        Technical Business Rule Format:
        ^TRT30 = NULL AND CountDocument('IEE') > 0

        Data Elements:
    
        ^TRT30 = TRT:RP:InterposedEntityElectionStatus:Elections.InterposedEntityElectionStatus.Year

        ^Count of IEE = ^Count of IEE
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432019()
        {
            bool assertion = parentDocument.TRT30 == null && ieeChildDocuments.Count() > 0;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.430178",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"IEE specified income year must be present",
                    LongDescription = @"If an IEE schedule is present, the Interposed entity election status - Specified income year must be present",
                    Location = "/tns:TRT/tns:RP/tns:InterposedEntityElectionStatus/tns:ElectionsY",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432019" },
                        new ProcessMessageParameter() { Name = "TRT30", Value = "null" },
                        new ProcessMessageParameter() { Name = "Count of IEE", Value = ieeChildDocuments.Count().ToString() },
                    },
                });
            }
        }
        #endregion // VR.ATO.TRT.432019

        #region VR.ATO.TRT.432021

        /*  VR.ATO.TRT.432021
        The Interposed Entity Election code indicates a revocation is being made, however no schedule is attached.

        Legacy Rule Format:
        ^TRT31 = 'R' AND CountDocument('IEE') = 0

        Technical Business Rule Format:
        ^TRT31 = 'R' AND CountDocument('IEE') = 0

        Data Elements:
    
        ^TRT31 = TRT:RP:InterposedEntityElectionStatus:Elections.InterposedEntityElectionRevocation.Code
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432021()
        {
            bool assertion = parentDocument.TRT31 == @"R" && !ieeChildDocuments.Any();

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.430180",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"If the Interposed Entity Election revocation code is present, an IEE schedule must be present",
                    Location = "/tns:TRT/tns:RP/tns:InterposedEntityElectionStatus/tns:ElectionsInterposedEntityElectionRevocationC",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432021" },
                        new ProcessMessageParameter() { Name = "TRT31", Value = "R" },
                    },
                });
            }
        }
        #endregion // VR.ATO.TRT.432021

        #region VR.ATO.TRT.432927

        /*  VR.ATO.TRT.432927
        An Entity cannot interpose itself; therefore the family trust tax file number quoted in Interposed Entity Election cannot be the same as the tax file number of the entity making the election

        Legacy Rule Format:
        CountDocument('IEE') > 0 AND AnyOccurrence(^IEE50, ^IEE50 = ^TRT5)

        Technical Business Rule Format:
        CountDocument('IEE') > 0 AND AnyOccurrence(^IEE50, ^IEE50 = ^TRT5)

        Data Elements:

        ^TRT5 = TRT:RP:Identifiers.TaxFileNumber.Identifier

        ^IEE50 = IEE:RP:Identifiers.TaxFileNumber.Identifier
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432927()
        {
            IEE2021 childDocument = null;

            if (ieeChildDocuments.Count() > 0)
            {
                childDocument = ieeChildDocuments.Any(iee => iee.IEE50 == parentDocument.TRT5) ? ieeChildDocuments.First(iee => iee.IEE50 == parentDocument.TRT5) : null;
            }

            bool assertion = childDocument != null;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.430008",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The Family Trust TFN on the IEE must not be the same as the main entity TFN",
                    LongDescription = @"An Entity cannot interpose itself; therefore the family trust tax file number quoted in Interposed Entity Election cannot be the same as the tax file number of the entity making the election",
                    Location = "/tns:TRT/tns:RP/tns:TaxFileNumberId",
                    Parameters = new ProcessMessageParameters {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432927" },
                        new ProcessMessageParameter { Name = "TRT5", Value = parentDocument.TRT5 },
                        new ProcessMessageParameter { Name = "IEE50", Value = childDocument.IEE50 }
                    }
                });
            }
        }

        #endregion // VR.ATO.TRT.432927

        #endregion //IEE CrossForm Validation Rules

        #region LS CrossForm Validation Rules

        #region VR.ATO.TRT.432208

        /*  VR.ATO.TRT.432208
        Public unit trust (listed other than cash management unit trust) that have Tax losses deducted of greater than $100,000 must complete Part B1 of the Losses schedule for unconsolidated entities. Trusts are required to indicate whether continuity of majority ownership test passed for each year of loss.

        Legacy Rule Format:
        ^TRT32 = '179' AND ^TRT144 > 100000 AND CountDocument('LS') = 1 AND ^LS15 = NULL AND ^LS16 = NULL AND ^LS17 = NULL AND ^LS18 = NULL AND ^LS19 = NULL AND ^LS53 = NULL

        Technical Business Rule Format:
        ^TRT32 = '179' AND ^TRT144 > 100000 AND CountDocument('LS') = 1 AND ^LS15 = NULL AND ^LS16 = NULL AND ^LS17 = NULL AND ^LS18 = NULL AND ^LS19 = NULL AND ^LS53 = NULL

        Data Elements:
    
        ^TRT144 = TRT:RP:BusinessDetails:Income:TaxLossesDeducted:IncomeTax.Deduction.TaxLossesDeducted.Amount

        ^TRT32 = TRT:RP:TypeOfTrust:OrganisationDetails.OrganisationType.Code
    
        ^LS15 = LS:RP:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
    
        ^LS16 = LS:RP:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
    
        ^LS17 = LS:RP:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
    
        ^LS18 = LS:RP:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
    
        ^LS19 = LS:RP:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
    
        ^LS53 = LS:RP:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
        
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432208(LS2017 childDocument)
        {
            bool assertion = parentDocument.TRT32 == @"179" && parentDocument.TRT144.GetValueOrDefault() > 100000 && childDocument.LS15 == null && childDocument.LS16 == null && childDocument.LS17 == null && childDocument.LS18 == null && childDocument.LS19 == null && childDocument.LS53 == null;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.432208",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Widely held trusts must indicate whether continuity of majority ownership test passed for each year of loss",
                    LongDescription = @"Public unit trust (listed other than cash management unit trust) that have Tax losses deducted of greater than $100,000 must complete Part B1 of the Losses schedule for unconsolidated entities",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:TaxLossesDeductedTaxDeductionA",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432208" },
                        new ProcessMessageParameter() { Name = "TRT144", Value = parentDocument.TRT144.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "TRT32", Value = "179" },
                        new ProcessMessageParameter() { Name = "LS15", Value = TRT2025Validator.GetValueOrEmpty(childDocument.LS15) },
                        new ProcessMessageParameter() { Name = "LS16", Value = TRT2025Validator.GetValueOrEmpty(childDocument.LS16) },
                        new ProcessMessageParameter() { Name = "LS17", Value = TRT2025Validator.GetValueOrEmpty(childDocument.LS17) },
                        new ProcessMessageParameter() { Name = "LS18", Value = TRT2025Validator.GetValueOrEmpty(childDocument.LS18) },
                        new ProcessMessageParameter() { Name = "LS19", Value = TRT2025Validator.GetValueOrEmpty(childDocument.LS19) },
                        new ProcessMessageParameter() { Name = "LS53", Value = TRT2025Validator.GetValueOrEmpty(childDocument.LS53) }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.432208

        #region VR.ATO.TRT.432210

        /*  VR.ATO.TRT.432210
        Where the total of tax losses AND net capital losses exceeds $100000, a Losses schedule must be attached.

        Legacy Rule Format:
        (^TRT145 + ^TRT475) > 100000 AND ^TRT44 <> TRUE AND CountDocument('LS') = 0

        Technical Business Rule Format:
        (^TRT145 + ^TRT475) > 100000 AND ^TRT44 <> TRUE AND CountDocument('LS') = 0

        Data Elements:

        ^TRT145 = TRT:RP:BusinessDetails:Income:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount

        ^TRT475 = TRT:RP:BusinessDetails:Income:Capital.Losses.CarriedForward.Net.Amount

        ^TRT44 = TRT:RP:BusinessDetails:OrganisationDetails.TaxConsolidationStatus.Indicator
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432210()
        {
            bool assertion = parentDocument.TRT145.GetValueOrDefault() + parentDocument.TRT475.GetValueOrDefault() > 100000 && parentDocument.TRT44 != true && !lsChildDocuments.Any();

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.432210",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Losses schedule must be attached where total of tax losses and net capital losses exceeds $100000",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:TaxLossesCarriedForwardLaterIncomeYearsTotalA",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432210" },
                        new ProcessMessageParameter() { Name = "TRT145", Value = parentDocument.TRT145.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "TRT475", Value = parentDocument.TRT475.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "TRT44", Value = parentDocument.TRT44.GetValueOrDefault().ToString() }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.432210

        #region VR.ATO.TRT.432212

        /*  VR.ATO.TRT.432212
        Total of the tax losses carried forward to later income years in the losses schedule for unconsolidated entities must equal tax losses carried forward to later income years on the main income tax return.

        Legacy Rule Format:
        CountDocument('LS') = 1 AND ^LS50 <> ^TRT145

        Technical Business Rule Format:
        CountDocument('LS') = 1 AND ^LS50 <> ^TRT145

        Data Elements:
    
        ^TRT145 = TRT:RP:BusinessDetails:Income:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount
    
        ^LS50 = LS:RP:Tax.Losses.CarriedForward.Total.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432212(LS2017 childDocument)
        {
            bool assertion = childDocument.LS50.GetValueOrDefault() != parentDocument.TRT145.GetValueOrDefault();

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.434086",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax losses carried forward to later income years in schedule must equal amount in main form",
                    LongDescription = @"The total of the 'Tax losses carried forward to later income years'  for unconsolidated entities in the Losses schedule must equal 'Tax losses carried forward to later income years' on the main form",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:TaxLossesCarriedForwardLaterIncomeYearsTotalA",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432212" },
                        new ProcessMessageParameter() { Name = "TRT145", Value = parentDocument.TRT145.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "LS50", Value = childDocument.LS50.GetValueOrDefault().ToString() }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.432212

        #region VR.ATO.TRT.432215

        /*  VR.ATO.TRT.432215
        Net capital losses carried forward to later income years - Total in the Losses schedule for unconsolidated entities is equal to the Net capital losses carried forward to later income years on the main income tax return.

        Legacy Rule Format:
        CountDocument('LS') = 1 AND ^LS52 <> ^TRT475

        Technical Business Rule Format:
        CountDocument('LS') = 1 AND ^LS52 <> ^TRT475

        Data Elements:
    
        ^TRT475 = TRT:RP:BusinessDetails:Income:Capital.Losses.CarriedForward.Net.Amount
    
        ^LS52 = LS:RP:Capital.Losses.CarriedForward.Net.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432215(LS2017 childDocument)
        {
            bool assertion = childDocument.LS52.GetValueOrDefault() != parentDocument.TRT475.GetValueOrDefault();

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.434087",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital losses carried forward to later income years must equal amount in Losses schedule",
                    LongDescription = @"The total of the 'Net capital losses carried forward to later income years'  for unconsolidated entities in the Losses schedule must equal 'Net capital losses carried forward to later income years' on the main form",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:CapitalLossesCarriedForwardNetA",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432215" },
                        new ProcessMessageParameter() { Name = "TRT475", Value = parentDocument.TRT475.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "LS52", Value = childDocument.LS52.GetValueOrDefault().ToString() }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.432215

        #region VR.ATO.TRT.432429

        /*  VR.ATO.TRT.432429
        Losses schedule must be attached

        Legacy Rule Format:
        ^TRT32 = '179' AND ^TRT144 > 100000 AND CountDocument('LS') = 0

        Technical Business Rule Format:
        ^TRT32 = '179' AND ^TRT144 > 100000 AND CountDocument('LS') = 0

        Data Elements:

        ^TRT32 = TRT:RP:TypeOfTrust:OrganisationDetails.OrganisationType.Code
    
        ^TRT144 = TRT:RP:BusinessDetails:Income:TaxLossesDeducted:IncomeTax.Deduction.TaxLossesDeducted.Amount
        
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432429()
        {
            bool assertion = parentDocument.TRT32 == @"179" && parentDocument.TRT144.GetValueOrDefault() > 100000 && !lsChildDocuments.Any();

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.432429",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Losses schedule must be attached",
                    LongDescription = @"Public unit trust (listed other than cash management unit trust) that have Tax losses deducted of greater than $100,000 must complete Part B1 of the Losses schedule for unconsolidated entities",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:TaxLossesDeductedTaxDeductionA",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432429" },
                        new ProcessMessageParameter() { Name = "TRT32", Value = "179" },
                        new ProcessMessageParameter() { Name = "TRT144", Value = parentDocument.TRT144.GetValueOrDefault().ToString() },
                    },
                });
            }
        }

        #endregion // VR.ATO.TRT.432429

        #endregion // LS CrossForm Validation Rules

        #region PSS CrossForm Validation Rules

        #region VR.ATO.TRT.432043

        /*  VR.ATO.TRT.432043
        Payment Summary schedule must be present if Gross payments where ABN not quoted is present

        Legacy Rule Format:
        (^TRT46 + ^TRT47) > 0 AND CountDocument('PSS') = 0

        Technical Business Rule Format:
        (^TRT46 + ^TRT47) > 0 AND CountDocument('PSS') = 0

        Data Elements:
    
        ^TRT46 = TRT:RP:BusinessDetails:Income:BusinessIncomeAndExpenses:IncomePrimaryProduction:Remuneration.ABNNotQuotedPaymentGross.Amount
    
        ^TRT47 = TRT:RP:BusinessDetails:Income:BusinessIncomeAndExpenses:IncomeNonPrimaryProduction:Remuneration.ABNNotQuotedPaymentGross.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432043()
        {
            bool assertion = parentDocument.TRT46.GetValueOrDefault() + parentDocument.TRT47.GetValueOrDefault() > 0 && !pssChildDocuments.Any();

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.430019",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Payment Summary schedule must be present if Gross payments where ABN not quoted is present",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:BusinessIncomeAndExpenses/tns:IncomePrimaryProduction/tns:RemunerationABNNotQuotedPaymentGrossA",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432043" },
                        new ProcessMessageParameter() { Name = "TRT46", Value = parentDocument.TRT46.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "TRT47", Value = parentDocument.TRT47.GetValueOrDefault().ToString() }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.432043

        #region VR.ATO.TRT.432044

        /*  VR.ATO.TRT.432044
        The sum of gross payment with indicator 'N' in the payment summary schedule must equal the sum of total gross payment where ABN not quoted for primary production AND total gross payment where ABN not quoted for non-primary production on the main return form

        Legacy Rule Format:
        CountDocument('PSS') > 0 AND Sum(^PSS19) <> (^TRT46 + ^TRT47)

        Technical Business Rule Format:
        CountDocument('PSS') > 0 AND Sum(^PSS19) <> (^TRT46 + ^TRT47)

        Data Elements:
    
        ^TRT46 = TRT:RP:BusinessDetails:Income:BusinessIncomeAndExpenses:IncomePrimaryProduction:Remuneration.ABNNotQuotedPaymentGross.Amount
    
        ^TRT47 = TRT:RP:BusinessDetails:Income:BusinessIncomeAndExpenses:IncomeNonPrimaryProduction:Remuneration.ABNNotQuotedPaymentGross.Amount
        
        ^Sum(PSS19) = Sum(^PSS19)
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432044()
        {
            bool assertion = pssChildDocuments.Any() && pssChildDocuments.Sum(f => f.RPPayerPSSeqNumContextCollection != null ? f.RPPayerPSSeqNumContextCollection.Sum(c => c.PSS19.GetValueOrDefault()) : 0) != (parentDocument.TRT46.GetValueOrDefault() + parentDocument.TRT47.GetValueOrDefault());
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.430020",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross payment where ABN not quoted must equal sum of payments on schedule",
                    LongDescription = @"The sum of gross payment with indicator 'N' in the Payment Summary schedule must equal the sum of Primary production and Non-primary production Gross payments where ABN not quoted on main return form",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:BusinessIncomeAndExpenses/tns:IncomePrimaryProduction/tns:RemunerationABNNotQuotedPaymentGrossA",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432044" },
                        new ProcessMessageParameter() { Name = "TRT46", Value = parentDocument.TRT46.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "TRT47", Value = parentDocument.TRT47.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "Sum(PSS19)", Value = pssChildDocuments.Sum(f => f.RPPayerPSSeqNumContextCollection != null ? f.RPPayerPSSeqNumContextCollection.Sum(c => c.PSS19.GetValueOrDefault()) : 0).ToString() }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.432044

        #region VR.ATO.TRT.432051

        /*  VR.ATO.TRT.432051
        The sum of gross payment with indicator 'F' in the Payment Summary schedule must equal total gross payment subject to foreign resident withholding (excluding capital gains) non-primary production on the main return form.

        Legacy Rule Format:
        CountDocument('PSS') > 0 AND ^TRT49 <> Sum(^PSS18)

        Technical Business Rule Format:
        CountDocument('PSS') > 0 AND ^TRT49 <> Sum(^PSS18)

        Data Elements:
    
        ^TRT49 = TRT:RP:BusinessDetails:Income:BusinessIncomeAndExpenses:IncomeNonPrimaryProduction:Remuneration.PaymentToForeignResidentGross.Amount
    
        ^Sum(^PSS18) = Sum(^PSS18)
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432051()
        {
            bool assertion = pssChildDocuments.Any() && parentDocument.TRT49.GetValueOrDefault() != pssChildDocuments.Sum(f => f.RPPayerPSSeqNumContextCollection != null ? f.RPPayerPSSeqNumContextCollection.Sum(c => c.PSS18.GetValueOrDefault()) : 0);
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.432749",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross payment - Foreign Resident Withholding (excluding capital gains) is incorrect",
                    LongDescription = @"The sum of gross payments with indicator 'F' in the Payment Summary schedule must be equal to total gross payment subject to foreign resident withholding (excluding capital gains) non-primary production on the main return form.",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:BusinessIncomeAndExpenses/tns:IncomeNonPrimaryProduction/tns:RemunerationPaymentToForeignResidentGrossA",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432051" },
                        new ProcessMessageParameter() { Name = "TRT49", Value = parentDocument.TRT49.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "Sum(PSS18)", Value = pssChildDocuments.Sum(f => f.RPPayerPSSeqNumContextCollection != null ? f.RPPayerPSSeqNumContextCollection.Sum(c => c.PSS18.GetValueOrDefault()) : 0).ToString() }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.432051

        #region VR.ATO.TRT.432088

        /*  VR.ATO.TRT.432088
        If an amount greater than zero has been entered at Tax withheld from payments where ABN not quoted, then a Payment Summary schedule must be present AND it must include the same total amount of Tax withheld from payments where ABN not quoted

        Legacy Rule Format:
        ^TRT73 > 0 AND (CountDocument('PSS') = 0 OR Sum(^PSS23) <> ^TRT73)

        Technical Business Rule Format:
        ^TRT73 > 0 AND (CountDocument('PSS') = 0 OR Sum(^PSS23) <> ^TRT73)

        Data Elements:
    
        ^TRT73 = TRT:RP:BusinessDetails:Income:PayAsYouGoWithholding:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
    
        ^Sum(PSS23) = Sum(PSS23)
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432088()
        {
            bool assertion = parentDocument.TRT73.GetValueOrDefault() > 0 && (!pssChildDocuments.Any() || pssChildDocuments.Sum(f => f.RPPayerPSSeqNumContextCollection != null ? f.RPPayerPSSeqNumContextCollection.Sum(c => c.PSS23.GetValueOrDefault()) : 0) != parentDocument.TRT73.GetValueOrDefault());

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.430183",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Payment Summary schedule must include Tax withheld where ABN not quoted amount",
                    LongDescription = @"If an amount greater than zero has been entered at Tax withheld from payments where ABN no quoted, then a Payment Summary schedule must be present and it must include the same total amount of Tax withheld from payments where ABN no quoted",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:PayAsYouGoWithholding/tns:TaxCreditForTaxWithheldWhereABNNotQuotedA",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432088" },
                        new ProcessMessageParameter() { Name = "TRT73", Value = parentDocument.TRT73.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "Sum(PSS23)", Value = pssChildDocuments.Sum(f => f.RPPayerPSSeqNumContextCollection != null ? f.RPPayerPSSeqNumContextCollection.Sum(c => c.PSS23.GetValueOrDefault()) : 0).ToString() }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.432088

        #endregion PSS CrossForm Validation Rules

        #region RS CrossForm Validation Rules

        #region VR.ATO.TRT.432115

        /*  VR.ATO.TRT.432115
        A rental schedule is attached AND rental income AND deduction amounts are not present in the return form.

        Legacy Rule Format:
        CountDocument('RS') > 0 AND ^TRT89 = NULL AND (^TRT90 = 0 OR ^TRT90 = NULL) AND (^TRT91 = 0 OR ^TRT91 = NULL) AND (^TRT92 = 0 OR ^TRT92 = NULL)

        Technical Business Rule Format:
        CountDocument('RS') > 0 AND ^TRT89 = NULL AND (^TRT90 = 0 OR ^TRT90 = NULL) AND (^TRT91 = 0 OR ^TRT91 = NULL) AND (^TRT92 = 0 OR ^TRT92 = NULL)

        Data Elements:
    
        ^TRT89 = TRT:RP:BusinessDetails:Income:Rent:Income.Operating.RentalIncomeGross.Amount
    
        ^TRT90 = TRT:RP:BusinessDetails:Income:Rent:Expense.Interest.Amount
    
        ^TRT91 = TRT:RP:BusinessDetails:Income:Rent:Expense.CapitalWorksDeduction.Amount
    
        ^TRT92 = TRT:RP:BusinessDetails:Income:Rent:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432115()
        {
            bool assertion = rsChildDocuments.Any() && parentDocument.TRT89 == null && (parentDocument.TRT90 == 0 || parentDocument.TRT90 == null) && (parentDocument.TRT91 == 0 || parentDocument.TRT91 == null) && (parentDocument.TRT92 == 0 || parentDocument.TRT92 == null);

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.430082",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross rent and Rental deduction amounts must be present",
                    LongDescription = @"If a Rental property schedule is present, Gross rent or rental deductions must be present",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:Rent/tns:OperatingIncomeGrossA",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432115" },
                        new ProcessMessageParameter() { Name = "TRT89", Value = "null" },
                        new ProcessMessageParameter() { Name = "TRT90", Value = "0" },
                        new ProcessMessageParameter() { Name = "TRT91", Value = "0" },
                        new ProcessMessageParameter() { Name = "TRT92", Value = "0" }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.432115

        #region VR.ATO.TRT.432116

        /*  VR.ATO.TRT.432116
        Total gross rent of schedules AND gross rent value in return must be equal

        Legacy Rule Format:
        CountDocument('RS') > 0 AND Sum(^RS38 + ^RS39) <> ^TRT89

        Technical Business Rule Format:
        CountDocument('RS') > 0 AND Sum(^RS38 + ^RS39) <> ^TRT89

        Data Elements:
    
        ^TRT89 = TRT:RP:BusinessDetails:Income:Rent:Income.Operating.RentalIncomeGross.Amount
    
        ^Sum(RS38 + RS39) = Sum(RS38 + RS39)
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432116()
        {
            bool assertion = rsChildDocuments.Any() && rsChildDocuments.Sum(rs => rs.RS38.GetValueOrDefault() + rs.RS39.GetValueOrDefault()) != parentDocument.TRT89.GetValueOrDefault();

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.430083",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross rent must equal the total Gross rent in the Rental Schedules",
                    LongDescription = @"The Gross rent amount in the main form must equal the total Gross rent present in all Rental property schedules",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:Rent/tns:OperatingIncomeGrossA",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432116" },
                        new ProcessMessageParameter() { Name = "TRT89", Value = parentDocument.TRT89.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "Sum(RS38 + RS39)", Value = rsChildDocuments.Sum(rs => rs.RS38.GetValueOrDefault() + rs.RS39.GetValueOrDefault()).ToString() }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.432116

        #region VR.ATO.TRT.432117

        /*  VR.ATO.TRT.432117
        Total net rent of schedule AND net rent value in return must be equal

        Legacy Rule Format:
        CountDocument('RS') > 0 AND Sum(^RS59) <> (^TRT89 - ^TRT90 - ^TRT91 - ^TRT92)

        Technical Business Rule Format:
        CountDocument('RS') > 0 AND Sum(^RS59) <> (^TRT89 - ^TRT90 - ^TRT91 - ^TRT92)

        Data Elements:
    
        ^TRT89 = TRT:RP:BusinessDetails:Income:Rent:Income.Operating.RentalIncomeGross.Amount
    
        ^TRT90 = TRT:RP:BusinessDetails:Income:Rent:Expense.Interest.Amount
    
        ^TRT91 = TRT:RP:BusinessDetails:Income:Rent:Expense.CapitalWorksDeduction.Amount
    
        ^TRT92 = TRT:RP:BusinessDetails:Income:Rent:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount

        Sum(RS59) = Sum(RS59)
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432117()
        {
            bool assertion = rsChildDocuments.Any() && rsChildDocuments.Sum(rs => rs.RS59.GetValueOrDefault()) != (parentDocument.TRT89.GetValueOrDefault() - parentDocument.TRT90.GetValueOrDefault() - parentDocument.TRT91.GetValueOrDefault() - parentDocument.TRT92.GetValueOrDefault());

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.430084",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net rent must equal the total net rent in the Rental Schedules",
                    LongDescription = @"The net rent amount in the main form (Gross rent minus rental deduction amounts) must equal the total Net rent present in all Rental property schedules",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:Rent/tns:OperatingIncomeGrossA",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432117" },
                        new ProcessMessageParameter() { Name = "TRT89", Value = parentDocument.TRT89.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "TRT90", Value = parentDocument.TRT90.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "TRT91", Value = parentDocument.TRT91.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "TRT92", Value = parentDocument.TRT92.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "Sum(RS59)", Value = rsChildDocuments.Sum(rs => rs.RS59.GetValueOrDefault()).ToString() }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.432117

        #region VR.ATO.TRT.432120

        /*  VR.ATO.TRT.432120
        If there is a positive amount at Gross rent or at any Rental deductions item, then a Rental property schedule must be submitted for each rental property

        Legacy Rule Format:
        (^TRT89 <> NULL OR ^TRT90 > 0 OR ^TRT91 > 0 OR ^TRT92 > 0) AND CountDocument('RS') = 0

        Technical Business Rule Format:
        (^TRT89 <> NULL OR ^TRT90 > 0 OR ^TRT91 > 0 OR ^TRT92 > 0) AND CountDocument('RS') = 0

        Data Elements:
    
        ^TRT89 = TRT:RP:BusinessDetails:Income:Rent:Income.Operating.RentalIncomeGross.Amount
    
        ^TRT90 = TRT:RP:BusinessDetails:Income:Rent:Expense.Interest.Amount
    
        ^TRT91 = TRT:RP:BusinessDetails:Income:Rent:Expense.CapitalWorksDeduction.Amount
    
        ^TRT92 = TRT:RP:BusinessDetails:Income:Rent:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432120()
        {
            bool assertion = (parentDocument.TRT89 != null || parentDocument.TRT90.GetValueOrDefault() > 0 || parentDocument.TRT91.GetValueOrDefault() > 0 || parentDocument.TRT92.GetValueOrDefault() > 0) && !rsChildDocuments.Any();

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.430081",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Rental schedule must be present",
                    LongDescription = @"If there is a positive amount at Gross rent or at any Rental deductions item, then a Rental property schedule must be submitted for each rental property",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:Rent/tns:OperatingIncomeGrossA",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432120" },
                        new ProcessMessageParameter() { Name = "TRT89", Value = parentDocument.TRT89.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "TRT90", Value = parentDocument.TRT90.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "TRT91", Value = parentDocument.TRT91.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "TRT92", Value = parentDocument.TRT92.GetValueOrDefault().ToString() }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.432120

        #region VR.ATO.TRT.432122

        /*  VR.ATO.TRT.432122
        Total interest of schedules AND interest value in return must be equal

        Legacy Rule Format:
        CountDocument('RS') > 0 AND Sum(^RS48) <> ^TRT90

        Technical Business Rule Format:
        CountDocument('RS') > 0 AND Sum(^RS48) <> ^TRT90

        Data Elements:
    
        ^TRT90 = TRT:RP:BusinessDetails:Income:Rent:Expense.Interest.Amount
    
        ^Sum(RS48) = Sum(RS48)
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432122()
        {
            bool assertion = rsChildDocuments.Any() && (rsChildDocuments.Sum(rs => rs.RS48.GetValueOrDefault()) != parentDocument.TRT90.GetValueOrDefault());

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.430086",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Rental Interest deductions must equal the total Interest deductions in the Rental Schedules",
                    LongDescription = @"The Interest deductions amount in the main form must equal the total of all Interest deductions in the Rental property schedules",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:Rent/tns:ExpenseInterestA",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432122" },
                        new ProcessMessageParameter() { Name = "TRT90", Value = parentDocument.TRT90.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "Sum(RS48)", Value = rsChildDocuments.Sum(rs => rs.RS48.GetValueOrDefault()).ToString() }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.432122

        #region VR.ATO.TRT.432124

        /*  VR.ATO.TRT.432124
        Total of capital works deductions claimed in attached schedules are not equal to the total of capital works deductions claimed in the return.

        Legacy Rule Format:
        CountDocument('RS') > 0 AND Sum(^RS54) <> ^TRT91

        Technical Business Rule Format:
        CountDocument('RS') > 0 AND Sum(^RS54) <> ^TRT91

        Data Elements:
    
        ^TRT91 = TRT:RP:BusinessDetails:Income:Rent:Expense.CapitalWorksDeduction.Amount
    
        ^Sum(RS54) = Sum(RS54)
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432124()
        {
            bool assertion = rsChildDocuments.Any() && (rsChildDocuments.Sum(rs => rs.RS54.GetValueOrDefault()) != parentDocument.TRT91.GetValueOrDefault());

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.430088",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital works deductions must equal the total Capital works deductions in the Rental Schedules",
                    LongDescription = @"The Capital works deductions amount in the main form must equal the total of all Capital works deductions in the Rental property schedules",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:Rent/tns:ExpenseCapitalWorksDeductionA",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432124" },
                        new ProcessMessageParameter() { Name = "TRT91", Value = parentDocument.TRT91.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "Sum(RS54)", Value = rsChildDocuments.Sum(rs => rs.RS54.GetValueOrDefault()).ToString() }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.432124

        #region VR.ATO.TRT.432126

        /*  VR.ATO.TRT.432126
        Total of rental expenses less Interest AND Capital works (Special building write-off) of schedules AND value of Other rental deductions in return must be equal

        Legacy Rule Format:
        CountDocument('RS') > 0 AND Sum(^RS40 + ^RS41 + ^RS42 + ^RS43 + ^RS44 + ^RS45 + ^RS46 + ^RS47 + ^RS49 + ^RS50 + ^RS51 + ^RS52 + ^RS53 + ^RS55 + ^RS56 + ^RS57 + ^RS58) <> ^TRT92

        Technical Business Rule Format:
        CountDocument('RS') > 0 AND Sum(^RS40 + ^RS41 + ^RS42 + ^RS43 + ^RS44 + ^RS45 + ^RS46 + ^RS47 + ^RS49 + ^RS50 + ^RS51 + ^RS52 + ^RS53 + ^RS55 + ^RS56 + ^RS57 + ^RS58) <> ^TRT92

        Data Elements:
    
        ^TRT92 = TRT:RP:BusinessDetails:Income:Rent:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
    
        ^Sum(RS Amounts) = Sum(RS Amounts)
        */
        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOTRT432126()
        {
            decimal rsTotal = rsChildDocuments.Sum(rs =>
            rs.RS40.GetValueOrDefault() +
            rs.RS41.GetValueOrDefault() +
            rs.RS42.GetValueOrDefault() +
            rs.RS43.GetValueOrDefault() +
            rs.RS44.GetValueOrDefault() +
            rs.RS45.GetValueOrDefault() +
            rs.RS46.GetValueOrDefault() +
            rs.RS47.GetValueOrDefault() +
            rs.RS49.GetValueOrDefault() +
            rs.RS50.GetValueOrDefault() +
            rs.RS51.GetValueOrDefault() +
            rs.RS52.GetValueOrDefault() +
            rs.RS53.GetValueOrDefault() +
            rs.RS55.GetValueOrDefault() +
            rs.RS56.GetValueOrDefault() +
            rs.RS57.GetValueOrDefault() +
            rs.RS58.GetValueOrDefault());

            bool assertion = rsChildDocuments.Any() && rsTotal != parentDocument.TRT92.GetValueOrDefault();

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.430090",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other rental deductions must equal the total other deductions in the Rental Schedules",
                    LongDescription = @"The Other rental deductions amount in the main form must equal the total of all rental expenses in the Rental property, excluding Interest deductions and Capital works deductions",
                    Location = "/tns:TRT/tns:RP/tns:BusinessDetails/tns:Income/tns:Rent/tns:TaxDeductionIncomeDeductionsOtherThanInterestAndCapitalWorksA",
                    Parameters = new ProcessMessageParameters() {
                        new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432126" },
                        new ProcessMessageParameter() { Name = "TRT92", Value = parentDocument.TRT92.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter() { Name = "Sum(RS Amounts)", Value = rsTotal.ToString() }
                    }
                });
            }
        }
        #endregion // VR.ATO.TRT.432126

        #endregion RS CrossForm Validation Rules
    }
}